package com.ngrob.android.bluemoon

import android.util.Log
import androidx.lifecycle.ViewModel
import com.ngrob.android.bluemoon.core.notifications.NotificationType
import com.ngrob.android.bluemoon.core.reminder.ReminderItem
import com.ngrob.android.bluemoon.core.reminder.ReminderNotificationScheduler
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.update
import java.time.LocalDate
import java.time.LocalDateTime
import javax.inject.Inject

data class MainActivityUiState(
    val selectedDay: LocalDate = LocalDate.now(),
)

@HiltViewModel
class BluemoonAppViewModel @Inject constructor(
    private val reminderNotificationScheduler: ReminderNotificationScheduler
) : ViewModel() {
    private val _uiState = MutableStateFlow(MainActivityUiState())
    val uiState = _uiState.asStateFlow()

    fun selectDay(newDay: LocalDate) {
        _uiState.update { currentState ->
            currentState.copy(
                selectedDay = newDay,
            )
        }
    }

    fun triggerNotification(){
        reminderNotificationScheduler.schedule(ReminderItem(LocalDateTime.now().plusSeconds(10), notificationType = NotificationType.PILL_REMINDER))
    }
}