package com.ngrob.android.bluemoon.features.edit_period.screens

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.ngrob.android.bluemoon.MenstruationCycleTracker
import com.ngrob.android.bluemoon.core.data.repository.BleedingRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.emptyFlow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.time.LocalDate
import javax.inject.Inject

data class EditPeriodUiState constructor(
    val bleedingDatesToDelete: List<LocalDate> = listOf(),
    val bleedingDays: Flow<List<LocalDate>> = emptyFlow(),
    val cycleTracker: MenstruationCycleTracker = MenstruationCycleTracker(listOf()),
    val saveButtonEnabled: Boolean = true
)

@HiltViewModel
class EditPeriodViewModel @Inject constructor(
    val bleedingRepository: BleedingRepository,
) : ViewModel() {

    private val _uiState = MutableStateFlow(EditPeriodUiState())
    val uiState = _uiState.asStateFlow()

    private var fetchBleeding: Job? = null

    init {
        getAllBleedingDates()
    }

    private fun getAllBleedingDates() {
        fetchBleeding?.cancel()
        fetchBleeding = viewModelScope.launch {
            val newBleedingDays = withContext(Dispatchers.IO) {
                bleedingRepository.getAllBleedingDates()
            }

            val newCycleTracker =
                MenstruationCycleTracker(newBleedingDays.stateIn(viewModelScope).value)
            _uiState.update { currentState ->
                currentState.copy(
                    bleedingDays = newBleedingDays,
                    cycleTracker = newCycleTracker
                )
            }
        }
    }

    fun onBleedingDayClicked(date: LocalDate) {

        viewModelScope.launch {
            _uiState.update { currentState ->
                val newBleedingDaysToDelete = currentState.bleedingDatesToDelete.toMutableList()

                val newBleedingDays = currentState.bleedingDays.map { latest ->
                    val newBleeding = latest.toMutableList()
                    if (date in latest) {
                        newBleeding.remove(date)
                        newBleedingDaysToDelete.add(date)
                    } else {
                        newBleeding.add(date)
                        if (date in newBleedingDaysToDelete){
                            newBleedingDaysToDelete.remove(date)

                        }
                    }
                    newBleeding.toList()
                }

                currentState.copy(
                    bleedingDays = newBleedingDays,
                    cycleTracker = MenstruationCycleTracker(newBleedingDays.stateIn(viewModelScope).value),
                    bleedingDatesToDelete = newBleedingDaysToDelete.toList()
                )
            }
        }
    }

    fun saveData() {
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                bleedingRepository.addBleedings(_uiState.value.bleedingDays.stateIn(viewModelScope).value)
                bleedingRepository.deleteBleedings(_uiState.value.bleedingDatesToDelete)
            }
        }
    }
}